package com.tuoheng.gateway.config;

import com.tuoheng.gateway.constants.AuthorityConstant;
import com.tuoheng.gateway.constants.PermitPathConstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.security.oauth2.resource.OAuth2ResourceServerProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.ReactiveAuthenticationManager;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.reactive.EnableWebFluxSecurity;
import org.springframework.security.config.web.server.ServerHttpSecurity;
import org.springframework.security.oauth2.jwt.NimbusReactiveJwtDecoder;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationConverter;
import org.springframework.security.oauth2.server.resource.authentication.JwtGrantedAuthoritiesConverter;
import org.springframework.security.oauth2.server.resource.authentication.JwtReactiveAuthenticationManager;
import org.springframework.security.oauth2.server.resource.authentication.ReactiveJwtAuthenticationConverterAdapter;
import org.springframework.security.web.server.SecurityWebFilterChain;

/**
 * @author chenjiandong
 * @description: TODO
 * @date 2022/9/28 13:53
 */
@Configuration
@EnableGlobalMethodSecurity(prePostEnabled = true)
@EnableWebFluxSecurity
public class WebSecurityConfig {

    /**
     * 需要登录后访问的接口
     */
    public static String oauthUrlStr;

    /**
     * 不需要登录就可以访问的接口
     */
    public static String permitUrlStr;

    @Value("${security.ignore.oauthUrls}")
    public void setOauthUrlStr(String oauthUrls) {
        oauthUrlStr = oauthUrls;
    }

    @Value("${security.ignore.permitUrls}")
    public void setPermitUrlStr(String permitUrls) {
        permitUrlStr = permitUrls;
    }

    private OAuth2ResourceServerProperties.Jwt Properties;

    @Autowired
    public void ResourceServerConfigurer(OAuth2ResourceServerProperties Properties) {
        this.Properties = Properties.getJwt();
    }

    @Bean
    public SecurityWebFilterChain securityWebFilterChain(ServerHttpSecurity httpSecurity) {
        String[] OAUTH_PATH = oauthUrlStr.split(",");
        String[] hhzPermitPath = PermitPathConstant.hhzPermitUrlStr;
        String[] freewayPermitPath = PermitPathConstant.freewayPermitUrlStr;
        String[] waterwayPermitPath = PermitPathConstant.waterwayPermitUrlStr;
        String[] dspInspectionPermitUrlStr = PermitPathConstant.dspInspectionPermitUrlStr;
        String[] weptspPermitPath = PermitPathConstant.weptspPermitUrlStr;
        String[] airmonitorPermitPath = PermitPathConstant.airmonitorPermitUrlStr;
        String[] telecomumalePermitPath = PermitPathConstant.telecomumaleUrlStr;
        String[] alertPermitPath = PermitPathConstant.alertleUrlStr;
        String[] airportPermitPath = PermitPathConstant.airportUrlStr;
        String[] oidcPermitPath = PermitPathConstant.oidcUrlStr;
        httpSecurity
                .authorizeExchange()
                //.pathMatchers(OAUTH_PATH).hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_DSP_MP, AuthorityConstant.SCOPE_TUOHNEG_DSP_WEB)
                .pathMatchers(hhzPermitPath).permitAll()
                .pathMatchers(freewayPermitPath).permitAll()
                .pathMatchers(waterwayPermitPath).permitAll()
                .pathMatchers(dspInspectionPermitUrlStr).permitAll()
                .pathMatchers(weptspPermitPath).permitAll()
                .pathMatchers(airmonitorPermitPath).permitAll()
                .pathMatchers(telecomumalePermitPath).permitAll()
                .pathMatchers(alertPermitPath).permitAll()
                .pathMatchers(airportPermitPath).permitAll()
                .pathMatchers(oidcPermitPath).permitAll()
                .pathMatchers("/pilot/miniprogram/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_PILOT_MP)
                .pathMatchers("/pilot/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_PILOT_ADMIN)
                .pathMatchers("/hhz/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_HHZ_ADMIN)
                .pathMatchers("/hhz/api/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_HHZ_MP)
                .pathMatchers("/freeway/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_FREEWAY_ADMIN)
                .pathMatchers("/freeway/miniprogram/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_FREEWAY_MP)
                .pathMatchers("/oidc/admin/tenant/**", "/oidc/admin/client/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_OIDC_ADMIN)
                .pathMatchers("/waterway/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_WATERWAY_ADMIN)
                .pathMatchers("/waterway/miniprogram/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_WATERWAY_MP)
                .pathMatchers("/telecomumale/miniprogram/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_TELECOMUMALE_MP)
                .pathMatchers("/telecomumale/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_TELECOMUMALE_ADMIN)
                .pathMatchers("/weptsp/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_WEPTSP_ADMIN)
                .pathMatchers("/airmonitor/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_AIRMONITOR_ADMIN)
                .pathMatchers("/alert/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_ALERT_ADMIN)
                .pathMatchers("/spacetime/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_SPACETIME_ADMIN)
                .pathMatchers("/digitaltwin/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_DIGITALTWIN_ADMIN)
                .pathMatchers("/dmp/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_DMP_ADMIN)
                //.pathMatchers("/lacs/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_LACS_ADMIN)
                .pathMatchers("/airport/admin/**").hasAnyAuthority(AuthorityConstant.SCOPE_ADMIN, AuthorityConstant.SCOPE_TUOHNEG_AIRPORT_ADMIN)
                .pathMatchers("/oidc/admin/user/**").authenticated()
                .pathMatchers("/api/inspection/**").authenticated()
                .pathMatchers(OAUTH_PATH).authenticated()
                //.pathMatchers(PERMIT_PATH).permitAll()
                .anyExchange().permitAll()
                .and()
                .csrf()
                .disable()
                .cors();
        httpSecurity.oauth2ResourceServer().jwt();
//        ServerHttpSecurity.OAuth2ResourceServerSpec.JwtSpec jwtSpec = httpSecurity.oauth2ResourceServer().jwt();
//        jwtSpec.authenticationManager(getAuthenticationManager());
        return httpSecurity.build();
    }

    ReactiveAuthenticationManager getAuthenticationManager() {
        NimbusReactiveJwtDecoder nimbusReactiveJwtDecoder = new NimbusReactiveJwtDecoder(Properties.getIssuerUri());
        JwtReactiveAuthenticationManager jwtReactiveAuthenticationManager = new JwtReactiveAuthenticationManager(nimbusReactiveJwtDecoder);

        JwtGrantedAuthoritiesConverter jwtGrantedAuthoritiesConverter = new JwtGrantedAuthoritiesConverter();
        jwtGrantedAuthoritiesConverter.setAuthorityPrefix("ROLE_");
        jwtGrantedAuthoritiesConverter.setAuthoritiesClaimName("authorities");

        JwtAuthenticationConverter jwtAuthenticationConverter = new JwtAuthenticationConverter();
        jwtAuthenticationConverter.setJwtGrantedAuthoritiesConverter(jwtGrantedAuthoritiesConverter);

        ReactiveJwtAuthenticationConverterAdapter reactiveJwtAuthenticationConverterAdapter = new ReactiveJwtAuthenticationConverterAdapter(jwtAuthenticationConverter);

        jwtReactiveAuthenticationManager.setJwtAuthenticationConverter(reactiveJwtAuthenticationConverterAdapter);
        return jwtReactiveAuthenticationManager;
    }

}
