package com.tuoheng.gateway.utils;

import com.alibaba.fastjson.JSONObject;
import com.tuoheng.gateway.commons.CommonsConfig;
import io.micrometer.core.instrument.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author chenjiandong
 * @description: 根据client-id获取路由path
 * @date 2022/11/17 9:38
 */
@Slf4j
public class GatewayUrlPathUtil {

    @Autowired
    private static RestTemplate restTemplate;

    private static final String HHZ_ADMIN = "tuoheng-hhz-admin";

    private static final String HHZ_MP = "tuoheng-hhz-mp";

    private static final String FREEWAY_ADMIN = "tuoheng-freeway-admin";

    private static final String FREEWAY_MP = "tuoheng-freeway-miniprogram";

    private static final String WATER_ADMIN = "tuoheng-waterway-admin";

    private static final String WATER_MP = "tuoheng-waterway-miniprogram";

    private static final String WSPTSP_ADMIN = "tuoheng-wsptsp-admin";

    private static final String AIRMONITOR_ADMIN = "tuoheng-airmonitor-admin";

    private static final String TELECOMUMALE_ADMIN = "tuoheng-telecomumale-admin";

    private static final String ALERT_ADMIN = "tuoheng-alert-admin";

    private static final String AIRPORT_ADMIN = "tuoheng-airport-admin";

    private static final String SPACETIME_ADMIN = "tuoheng-spacetime-admin";

    private static final String DIGITALTWIN_ADMIN = "tuoheng-digitaltwin-admin";

    private static final String DMP_ADMIN = "tuoheng-dmp-admin";

    private static final String LACS_ADMIN = "tuoheng-lacs-admin";


    /**
     * 获取 gateway 路由前缀，匹配url
     *
     * @param clientId
     * @return
     */
    public static String getPathByClientId(String clientId) {
        String apiPath = "";
        switch (clientId) {
            case HHZ_ADMIN:
                apiPath = "/hhz/admin";
                break;
            case HHZ_MP:
                apiPath = "/hhz/api";
                break;
            case FREEWAY_ADMIN:
                apiPath = "/freeway/admin";
                break;
            case FREEWAY_MP:
                apiPath = "/freeway/miniprogram";
                break;
            case WATER_ADMIN:
                apiPath = "/waterway/admin";
                break;
            case AIRPORT_ADMIN:
                apiPath = "/airport/admin";
                break;
            case WATER_MP:
                apiPath = "/waterway/miniprogram";
                break;
            case WSPTSP_ADMIN:
                apiPath = "/wsptsp/admin";
                break;
            case AIRMONITOR_ADMIN:
                apiPath = "/airmonitor/admin";
                break;
            case TELECOMUMALE_ADMIN:
                apiPath = "/telecomumale/admin";
                break;
            case ALERT_ADMIN:
                apiPath = "/alert/admin";
                break;
            case SPACETIME_ADMIN:
                apiPath = "/spacetime/admin";
                break;
            case DIGITALTWIN_ADMIN:
                apiPath = "/digitaltwin/admin";
                break;
            case DMP_ADMIN:
                apiPath = "/dmp/admin";
                break;
            case LACS_ADMIN:
                apiPath = "/lacs/admin";
                break;
            default:
                break;
        }
        return apiPath;
    }

    /**
     * 根据 clientId 从业务系统获取 permission - role 数据
     *
     * @return
     */
    public static List<Integer> getRoleIdByApiUrlPermission(String clientId, String apiUrl, String token) {
        log.info("getRoleIdByApiUrlPermission -start, clientId:{},apiUrl:{}", clientId, apiUrl);
        List<Integer> resList = new ArrayList<>();
        String url = "";
        if (clientId.equals(HHZ_ADMIN) || clientId.equals(HHZ_MP)) {
            url = CommonsConfig.hhzPermissionUrl;
        }
        if (clientId.equals(FREEWAY_ADMIN) || clientId.equals(FREEWAY_MP)) {
            url = CommonsConfig.freewayPermissionUrl;
        }
        if (clientId.equals(WATER_ADMIN) || clientId.equals(WATER_MP)) {
            url = CommonsConfig.waterwayPermissionUrl;
        }
        if (clientId.equals(WSPTSP_ADMIN)) {
            url = CommonsConfig.weptspPermissionUrl;
        }
        if (clientId.equals(AIRMONITOR_ADMIN)) {
            url = CommonsConfig.airmonitorPermissionUrl;
        }
        if (clientId.equals(TELECOMUMALE_ADMIN)) {
            url = CommonsConfig.telecomumalePermissionUrl;
        }
        if (clientId.equals(ALERT_ADMIN)) {
            url = CommonsConfig.alertPermissionUrl;
        }
        if (clientId.equals(AIRPORT_ADMIN)) {
            url = CommonsConfig.airportPermissionUrl;
        }
        if (clientId.equals(SPACETIME_ADMIN)) {
            url = CommonsConfig.spacetimePermissionUrl;
        }
        if (clientId.equals(DIGITALTWIN_ADMIN)) {
            url = CommonsConfig.digitaltwinPermissionUrl;
        }
        if (clientId.equals(DMP_ADMIN)) {
            url = CommonsConfig.dmpPermissionUrl;
        }
        if (clientId.equals(LACS_ADMIN)) {
            url = CommonsConfig.lacsPermissionUrl;
        }
        log.info("getRoleIdByApiUrlPermission -url:{},token:{}", url, token);
        if (StringUtils.isNotBlank(url)) {
            HttpHeaders resultRequestHeader = new HttpHeaders();
            resultRequestHeader.add("Authorization", "Bearer " + token);
            JSONObject json = new JSONObject();
            json.put("apiUrl", apiUrl);
            HttpEntity<JSONObject> entity = new HttpEntity<>(json, resultRequestHeader);
            String result = new RestTemplate().postForObject(url, entity, String.class);
            log.info("getRoleIdByApiUrlPermission - result:{}", result);
            JSONObject jsonObject = JSONObject.parseObject(result);
            Object obj = jsonObject.get("data");
            if (!Objects.isNull(obj)) {
                for (Object o : (List<?>) obj) {
                    resList.add(Integer.class.cast(o));
                }
            }
        }
        return resList;
    }

}
